from a2a.types import AgentExtension

PROTOCOL_URI = "https://gitlab.eclipse.org/eclipse-research-labs/mosaico-project/a2a-acl/-/blob/main/a2a_acl_protocol/MOSAICO_A2A_ACL_PROTOCOL"
EXTENSION = AgentExtension(
    uri=PROTOCOL_URI, description="MOSAICO A2A ACL", required=True
)
